
//+------------------------------------------------------------------+
//|                                                     TZ-Pivot.mq4 |
//|                                                                  |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Copyright Shimodax"
#property link      "http://www.strategybuilderfx.com"

/*-------------------------------------------------------------------------------
Introduction:

   Calculation of pivot and similar levels based on time zones.
   If you want to modify the colors, please scroll down to line
   200 and below (where it says "Calculate Levels") and change
   the colors.  Valid color names can be obtained by placing
   the curor on a color name (e.g. somewhere in the word "Orange"
   and pressing F1).
   
   Time-Zone Inputs:

   LocalTimeZone: TimeZone for which MT4 shows your local time, 
                  e.g. 1 or 2 for Europe (GMT+1 or GMT+2 (daylight 
                  savings time).  Use zero for no adjustment.                 
                  The MetaQuotes demo server uses GMT +2.
                  
   DestTimeZone:  TimeZone for the session from which to calculate
                  the levels (e.g. 1 or 2 for the European session
                  (without or with daylight savings time).  
                  Use zero for GMT
                             
   Example: If your MT server is living in the EST (Eastern Standard Time, 
            GMT-5) zone and want to calculate the levels for the London trading
            session (European time in summer GMT+1), then enter -5 for 
            LocalTimeZone, 1 for Dest TimeZone. 
            
            Please understand that the LocalTimeZone setting depends on the
            time on your MetaTrader charts (for example the demo server 
            from MetaQuotes always lives in CDT (+2) or CET (+1), no matter
            what the clock on your wall says. If in doubt, leave everything to zero.

   The following doesn't work yet, please leave it to 0/24:                 
            TradingHoursFrom: First hour of the trading session in the destination
            time zone.                    
            TradingHoursTo: Last hour of the trading session in the destination
            time zone (the hour starting with this value is excluded, i.e. 18 means
            up to 17:59 o'clock)
                   
   Example: If you are living in the EST (Eastern Standard Time, GMT-5) 
            zone and want to calculate the levels for the London trading
            session (European time GMT+1, 08:00 - 17:00), then enter
            -5 for LocalTimeZone, 1 for Dest TimeZone, 8 for HourFrom
            and 17 for hour to.
--------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------
SDX-TzPivots_v2:

This is a "cut and dice" upgrade....had to rip the guts out, clean house, and put
back just the essentials....making sure ALL that was put back ACTUALLY WORKED!
Some improvements were incorporated, too.  Shimodax made the start of a neat Indicator
to draw not only basic pivots, but also SweetSpots (for which another indicator,
SDX-SweetSpots, is available), more fibo lines of a different type of calculation,
and Camarilla lines.  But some of the "technical operations in the background" are
not functioning.  For example, the BarForLabels function does not work.  Some of the
formulations produce incorrect results. For example, the ShowHighLowOpen showed 
Yesterdays High for the user's selected session, but showed Yesterdays Low for the
MT4 server timezone session!  In addition, non-standard formulas are used to calculate
the basic pivot Support and Resistance lines.

Going far enough to uncover these problems, I decided to strip it down to just the
basic pivots, code in the standard formulas for the pivots, put all the support
operations in order, and add some enhancements.  Here are the indicator
window inputs and some comments........

HoursMT4TimeZoneFromGMT:
     Enter the number of hours difference between the MT4 server time zone, and GMT.
     For example, FXDD uses a server that is two hours before GMT, hence enter "2".
     If the server is in NY, 5 hours after GMT, you would enter "-5" ("-4" during
     DST).  Use default value "0" for normal, non-time shifted pivot calculations.
     
HoursSelectedTimeZonefromGMT:
     Enter the number of hours difference between your selected time zone to start
     the day (as regards the indicator's calculations), and GMT.  For example, if
     you wish to have the day start at NY time, then enter "-5".  If you wish to 
     have the day start at Zurich time, then enter "1".  Use the default value of
     "0" for normal, non-time shifted pivot calculations. 

Resistance/Central/Support Pivots:
     Enter the colors of your choice.  This is an enhancement.
     
LineStyle_01234:
     Your number entry selects the line style for the lines.  0=solid, 1=dash,
     2=dashdot, and 3=dashdotdot.
     
SolidLineThickness:
     Your number entry selects the width of solid lines.  0 and 1 = single width.
     2, 3, 4 graduate the thickness.  Enhanced coding assures that no matter what
     number this is set at, non-solid line styles selected will still display.

NewPeriodSeparatorsColor:
     Enter the color of your choice.  This is an enhancement.
          
ShowNewPeriodSeparators:
     The choice of "true" will place a pair of vertical lines on the chart showing the
     start and stop of the new 24 hour period you have selected for pivot calculations.
     Choosing "false" turns the pair off.  This is an enhancement.

SeparatorLineStyle_01234
     Enhanced coding allows you to select any line style for this pair of lines.

LineLabelColor:
     Enter the color of your choice.  This is an enhancement.
     
LineLabelFontSize:
     Change the font size of the labels.  This is an enhancement.     
     
LineLabelsIncludePrice:
     Lines are always labeled R#, PV, or S#, but selecting "true" will add the
     price.  Even if the price is not displayed, it always appears in the right margin
     of the chart.....another enhancement.                   

MoveLabels_LR_IncrDecr:
     Increasing the number moves the line labels to the left on the chart.
     Decreasing the number moves the line labels to the right on the chart.
     This feature is inoperative in the first indicator.
     
ShowComment:
     If "true" then key prior day and current day data appears in upper left of chart.

The original indicator does not extend lines completely across the screen, and for some
of them this is a good feature.  But the basic pivot lines are more useful if they are
extended across the entire screen, which is the way they are coded now, in this
version, SDX-TzPivots_v2.
  
                                                   - Traderathome, November 17, 2008 
-------------------------------------------------------------------------------------*/



/*------------------------------------------------------------------------------------
SDX-TzPivots_v3:

My apologies to Shimdax for saying a non-standard formula is used in the original.
The formula used is not the standard formula for Fibonacci Pivots, but it is the
standard formula for Daily Pivots.  Both formulas are resident in this revision which
the user can select in the indicator window.   All the main pivot lines individually
have color, style, and solid line thickness selection (overrides non-solid lines
when greater than "1").  The Central Pivot line label shows "Daily" or "Fibonacci", 
depending on which was chosen.  The MidPivots feature is added back and now includes 
line style and color selections.  The feature to select lines that do not display 
across the entire screen is added back, but may still show lines across the screen 
on small timeframe charts when the "startofday" for calculations is already off the  
left side.  Coding of lines and labels is enchanced.  The selection to NOT display 
across the entire screen assures the lines and labels cannot exceed (to the left) 
the "today" Period Separator, regardless of a higher "MoveLabel..." setting. Period
Separator labels "Yesterday" and "Today" now have color selection. 


HoursMT4TimeZoneFromGMT:
     Enter the number of hours difference between the MT4 server time zone, and GMT.
     For example, FXDD uses a server that is two hours before GMT, hence enter "2".
     If the server is in NY, 5 hours after GMT, you would enter "-5" ("-4" during
     DST).  Use default value "0" for normal, non-time shifted pivot calculations.
     
HoursSelectedTimeZonefromGMT:
     Enter the number of hours difference between your selected time zone to start
     the day (as regards the indicator's calculations), and GMT.  For example, if
     you wish to have the day start at NY time, then enter "-5".  If you wish to 
     have the day start at Zurich time, then enter "1".  Use the default value of
     "0" for normal, non-time shifted pivot calculations. 
     
Show_1Daily_2FibonacciPivots:
     Either formula can be used to produce the main pivot lines.
     
Resistance/Central/Support Pivots:
     Coding enhancement allows user to select color, style and solid line thickness
     individually.  Style entries:  0=solid, 1=dash, 2=dashdot, and 3=dashdotdot.
     A thickness setting greater than "1" will override any non-solid style setting
     and display a solid line of that thickness selected.
     
MidPivots:
     Select color and style for these lines and turn them on or off.   

YesterdayTodayLabelsColor:
     Color the "Yesterday" and "Today" labels for the Period Separator lines.
     These labels will display when the Period Separators are selected.
     
PeriodSeparators:
     Enter the color and line style of choice for the Period Separators.
     The choice of "true" will place a pair of vertical lines on the chart showing
     the start and stop of the new 24 hour period you have selected for calculating
     pivots.  Choosing "false" cancels display of the pair of lines and their
     "Yesterday" and "Today" labels.

PivotLineLabels:
     Enter the color and font size of your choice.
     
FullLinesWithMarginLabels:
     "true" displays lines across entire screen with labels in the right margin.
     "false" displays lines starting at the "Today" Period Separator.
      
PivotLineLabelsIncludePrice:
     Lines are always labeled R#, PV, or S#, but selecting "true" will add the
     price.  Remember, on full screen lines, even if the price is not displayed
     it appears in the line label in the right margin of the chart.                 

MoveLabels_LR_IncrDecr:
     Increasing the number moves the line labels to the left on the chart.
     Decreasing the number moves the line labels to the right on the chart.
     This feature is inoperative in the original indicator.  If "false" is selected
     for "FullLinesWithMarginLabels", the labels will only display to the right of
     the "Today" Period Separator, no matter how large a number is input.
     
ShowComment:
     If "true" then key prior day and current day data appears in upper left of chart.
     

SDX-TzPivots_v3.1:

A missing code line in Deinitialization was added back.  Added two extra levels of R&S,
as well as the corresponding Midpivot lines. These extra levels are not in the indicator
window.  They will automatically display per the color of R3/S3 but with extra and 
increasing line thickness. If R3/S3 are single thickness non-solid lines, then R4/S4 
will be solid with thickness 1, R5/S5 will be solid thickness 2. If R3/S3 are solid lines 
with single thickness, then R4/S4 will be thickness 2, R5/S5 thickness 3.  Another 
change is that line styles selected other than solid are NOT overridden when the solid 
thickness setting is greater than 1, they will display anyway. When the line style is 
reselected to solid, the solid line will be per the existing thickness setting. 

                                                   - Traderathome, November 24, 2008
--------------------------------------------------------------------------------------*/

#property indicator_chart_window
extern int   HoursMT4TimeZoneFromGMT        = 2;
extern int   HoursSelectedTimeZonefromGMT   = -5;
extern int   Show_1Daily_2FibonacciPivots   = 2;
//extern color R4_Color                       = Red;
//extern int   R4_LineStyle_01234             = 2;
//extern int   R4_SolidLineThickness          = 3; 
extern color R3_Color                       = Red;
extern int   R3_LineStyle_01234             = 2;
extern int   R3_SolidLineThickness          = 2; 
extern color R2_Color                       = Crimson;
extern int   R2_LineStyle_01234             = 2;
extern int   R2_SolidLineThickness          = 2; 
extern color R1_Color                       = FireBrick;
extern int   R1_LineStyle_01234             = 2;
extern int   R1_SolidLineThickness          = 2; 
extern color CentralPivotColor              = Blue;
extern int   CentralPivotLineStyle_01234    = 0;
extern int   CentralPivotSolidLineThickness = 2; 
extern color S1_Color                       = DarkGreen;
extern int   S1_LineStyle_01234             = 2;
extern int   S1_SolidLineThickness          = 2; 
extern color S2_Color                       = Green;
extern int   S2_LineStyle_01234             = 2;
extern int   S2_SolidLineThickness          = 2; 
extern color S3_Color                       = ForestGreen;
extern int   S3_LineStyle_01234             = 2;
extern int   S3_SolidLineThickness          = 2; 
//extern color S4_Color                       = ForestGreen;
//extern int   S4_LineStyle_01234             = 2;
//extern int   S4_SolidLineThickness          = 3; 
extern color MidPivotsColor                 = White;
extern int   MidPivotsLineStyle_01234       = 2;
extern bool  ShowMidPivots                  = true;
extern color YesterdayTodayPeriodLabels     = Purple;
extern color PeriodSeparatorLines           = Yellow;
extern int   SeparatorLineStyle_01234       = 0;
extern bool  ShowPeriodSeparators           = true;
extern color PivotLinesLabelColor           = Navy;
extern int   LineLabelFontSize              = 9;
extern bool  FullLinesWithMarginLabels      = false;
extern bool  LineLabelShowsPrice            = true;
extern int   MoveLabel_LR_IncrDecr          = 50;  //+ to move left, - to move right
extern bool  ShowComment                    = false;
//int TradingHoursFrom= 0;
//int TradingHoursTo= 24;
int digits; //decimal digits for symbol's price       

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
	deinit();
	if (Ask>10) digits=2; else digits=4;
   Print("Period= ", Period());
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator de-initialization function                      |
//+------------------------------------------------------------------+
int deinit()
{
   int obj_total= ObjectsTotal();
   string gvname;   
   for (int i= obj_total; i>=0; i--)
      {
      string name= ObjectName(i);   
      if (StringSubstr(name,0,7)=="[PIVOT]")  ObjectDelete(name);
      }    
    	gvname=Symbol()+"p";
   	GlobalVariableDel(gvname);
   	gvname=Symbol()+"r1";
   	GlobalVariableDel(gvname);
   	gvname=Symbol()+"r2";
   	GlobalVariableDel(gvname);
   	gvname=Symbol()+"r3";
   	GlobalVariableDel(gvname);
      gvname=Symbol()+"r4";
   	GlobalVariableDel(gvname);
      gvname=Symbol()+"r5";
   	GlobalVariableDel(gvname);
   	gvname=Symbol()+"s1";
   	GlobalVariableDel(gvname);
   	gvname=Symbol()+"s2";
   	GlobalVariableDel(gvname);
   	gvname=Symbol()+"s3";  
      GlobalVariableDel(gvname);
      gvname=Symbol()+"s4";  
      GlobalVariableDel(gvname);
      gvname=Symbol()+"s5";  
      GlobalVariableDel(gvname);
      gvname=Symbol()+"mr5";
   	GlobalVariableDel(gvname);
   	gvname=Symbol()+"mr4";
   	GlobalVariableDel(gvname);
      gvname=Symbol()+"mr3";
   	GlobalVariableDel(gvname);
   	gvname=Symbol()+"mr2";
   	GlobalVariableDel(gvname);
   	gvname=Symbol()+"mr1";
   	GlobalVariableDel(gvname);
   	gvname=Symbol()+"ms1";
   	GlobalVariableDel(gvname);
   	gvname=Symbol()+"ms2";
   	GlobalVariableDel(gvname);
   	gvname=Symbol()+"ms3";
   	GlobalVariableDel(gvname);
      gvname=Symbol()+"ms4";
   	GlobalVariableDel(gvname);
   	gvname=Symbol()+"ms5";
   	GlobalVariableDel(gvname);
      Comment(" ");
   return(0);
}
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{  
   static datetime timelastupdate= 0;
   static datetime lasttimeframe= 0;  
   datetime startofday= 0,
            startofyesterday= 0;
   double today_high= 0,
            today_low= 0,
            today_open= 0,
            yesterday_high= 0,
            yesterday_open= 0,
            yesterday_low= 0,
            yesterday_close= 0;
   int idxfirstbaroftoday= 0,
       idxfirstbarofyesterday= 0,
       idxlastbarofyesterday= 0;  
   // no need to update these buggers too often   
   if (CurTime()-timelastupdate<60 && Period()==lasttimeframe)  return (0);     
   lasttimeframe= Period();
   timelastupdate= CurTime();   
   //---- exit if period is greater than daily charts
   if(Period() > 1440) {
      Alert("Error - Chart period is greater than 1 day.");
      return(-1); // then exit
   }  
   string gvname; double gvval;
   // let's find out which hour bars make today and yesterday
   ComputeDayIndices(HoursMT4TimeZoneFromGMT, HoursSelectedTimeZonefromGMT, idxfirstbaroftoday, idxfirstbarofyesterday, idxlastbarofyesterday);
   startofday= Time[idxfirstbaroftoday+28];  // datetime (x-value) for labes on horizontal bars
   gvname=Symbol()+"st";
   gvval=startofday;
   GlobalVariableSet(gvname,gvval);
   startofyesterday= Time[idxfirstbarofyesterday+28];  // datetime (x-value) for labes on horizontal bars
   // walk forward through yestday's start and collect high/lows within the same day
   yesterday_high= -99999;  // not high enough to remain alltime high
   yesterday_low=  +99999;  // not low enough to remain alltime low   
   for (int idxbar= idxfirstbarofyesterday; idxbar>=idxlastbarofyesterday; idxbar--) {
      if (yesterday_open==0)  // grab first value for open
         yesterday_open= Open[idxbar];                           
         yesterday_high= MathMax(High[idxbar], yesterday_high);
         yesterday_low= MathMin(Low[idxbar], yesterday_low);      
   // overwrite close in loop until we leave with the last iteration's value
         yesterday_close= Close[idxbar];
   }
   // walk forward through today and collect high/lows within the same day
   today_open= Open[idxfirstbaroftoday];  // should be open of today start trading hour
   today_high= -99999; // not high enough to remain alltime high
   today_low=  +99999; // not low enough to remain alltime low
   for (int j= idxfirstbaroftoday; j>=0; j--) {
      today_high= MathMax(today_high, High[j]);
      today_low= MathMin(today_low, Low[j]);
   }       
   // draw the vertical bars that marks the time span
   double level= (yesterday_high + yesterday_low + yesterday_close) / 3;
   SetTimeLine("YesterdayStart", "Yesterday", idxfirstbarofyesterday, YesterdayTodayPeriodLabels, level + 50*Point);
   SetTimeLine("YesterdayEnd", "Today", idxfirstbaroftoday, YesterdayTodayPeriodLabels, level + 50*Point);       
  
   //---- Calculate Pivot Levels
   double p, q, d, r1,r2,r3,r4,r5, s1,s2,s3,s4,s5;   
   d = (today_high - today_low);
   q = (yesterday_high - yesterday_low);
   p = (yesterday_high + yesterday_low + yesterday_close) / 3;

   if(Show_1Daily_2FibonacciPivots == 1) 
   {
   r1 = (2*p)-yesterday_low;
   r2 = p+(yesterday_high - yesterday_low);  //r2 = p-s1+r1;
   r3 = (2*p)+(yesterday_high-(2*yesterday_low));
   s1 = (2*p)-yesterday_high;
   s2 = p-(yesterday_high - yesterday_low);  //s2 = p-r1+s1;
   s3 = (2*p)-((2* yesterday_high)-yesterday_low);
   }

   if(Show_1Daily_2FibonacciPivots == 2)
   {
   r1 = p+ (q * 0.382);   
   r2 = p+ (q * 0.618);  
 	r3 = p+q;  
   r4 = p+ (q * 1.618);
   r5 = p+ (q * 2.618);
   s1 = p- (q * 0.382); 
   s2 = p- (q * 0.618);   
  	s3 = p-q; 
   s4 = p- (q * 1.618);
   s5 = p- (q * 2.618);
   }
     
   p=NormalizeDouble(p,digits);
   gvname=Symbol()+"p";
   gvval=p;
   GlobalVariableSet(gvname,gvval);    
   r1=NormalizeDouble(r1,digits);
   gvname=Symbol()+"r1";
   gvval=r1;
   GlobalVariableSet(gvname,gvval);  
   r2=NormalizeDouble(r2,digits);
   gvname=Symbol()+"r2";
   gvval=r2;
   GlobalVariableSet(gvname,gvval);
   r3=NormalizeDouble(r3,digits);
   gvname=Symbol()+"r3";
   gvval=r3;
   GlobalVariableSet(gvname,gvval); 
   r4=NormalizeDouble(r4,digits);
   gvname=Symbol()+"r4";
   gvval=r4;
   GlobalVariableSet(gvname,gvval);
   r5=NormalizeDouble(r5,digits);
   gvname=Symbol()+"r5";
   gvval=r5;
   GlobalVariableSet(gvname,gvval);
   s1=NormalizeDouble(s1,digits);
   gvname=Symbol()+"s1";
   gvval=s1;
   GlobalVariableSet(gvname,gvval);
   s2=NormalizeDouble(s2,digits);
   gvname=Symbol()+"s2";
   gvval=s2;
   GlobalVariableSet(gvname,gvval);
   s3=NormalizeDouble(s3,digits);
   gvname=Symbol()+"s3";
   gvval=s3;
   GlobalVariableSet(gvname,gvval);
   s4=NormalizeDouble(s4,digits);
   gvname=Symbol()+"s4";
   gvval=s4;
   GlobalVariableSet(gvname,gvval);
   s5=NormalizeDouble(s5,digits);
   gvname=Symbol()+"s5";
   gvval=s5;
   GlobalVariableSet(gvname,gvval);
 
   startofday = Time[idxfirstbaroftoday];        
      {
      if(R3_LineStyle_01234>0){R3_SolidLineThickness=0;} 
      if(R2_LineStyle_01234>0){R2_SolidLineThickness=0;}
      if(R1_LineStyle_01234>0){R1_SolidLineThickness=0;} 
      if(S1_LineStyle_01234>0){S1_SolidLineThickness=0;}
      if(S2_LineStyle_01234>0){S2_SolidLineThickness=0;} 
      if(S3_LineStyle_01234>0){S3_SolidLineThickness=0;}
      
      SetLevel("R1", r1, R1_Color, R1_LineStyle_01234, R1_SolidLineThickness, startofday);
      SetLevel("R2", r2, R2_Color, R2_LineStyle_01234, R2_SolidLineThickness, startofday);
      SetLevel("R3", r3, R3_Color, R3_LineStyle_01234, R3_SolidLineThickness, startofday);      
      SetLevel("S1", s1, S1_Color, S1_LineStyle_01234 , S1_SolidLineThickness, startofday);
      SetLevel("S2", s2, S2_Color, S2_LineStyle_01234 , S2_SolidLineThickness, startofday);
      SetLevel("S3", s3, S3_Color, S3_LineStyle_01234 , S3_SolidLineThickness, startofday); 
      if(Show_1Daily_2FibonacciPivots == 1){     
      SetLevel("Daily Pivot", p, CentralPivotColor, CentralPivotLineStyle_01234 ,
               CentralPivotSolidLineThickness, startofday);}
      if(Show_1Daily_2FibonacciPivots == 2){  
      SetLevel("      Fibonacci Pivot", p, CentralPivotColor, CentralPivotLineStyle_01234 ,
               CentralPivotSolidLineThickness, startofday);     
      if (R3_LineStyle_01234 == 0 && R3_SolidLineThickness <= 1) {R3_SolidLineThickness =1;}
      if (S3_LineStyle_01234 == 0 && S3_SolidLineThickness <= 1) {S3_SolidLineThickness =1;}
      SetLevel("R4", r4, R3_Color, STYLE_SOLID, R3_SolidLineThickness+1, startofday); 
      SetLevel("R5", r5, R3_Color, STYLE_SOLID, R3_SolidLineThickness+2, startofday);    
      SetLevel("S4", s4, S3_Color, STYLE_SOLID, S3_SolidLineThickness+1, startofday);
      SetLevel("S5", s5, S3_Color, STYLE_SOLID, S3_SolidLineThickness+2, startofday);}
      }
   
   //------ Midpoints Pivots (mid-levels between pivots)
   if (ShowMidPivots==true) {
      int LineStyle=MidPivotsLineStyle_01234; int LineThickness=0;
      SetLevel("MR5", (r4+r5)/2,    MidPivotsColor, LineStyle, LineThickness, startofday);
      SetLevel("MR4", (r3+r4)/2,    MidPivotsColor, LineStyle, LineThickness, startofday);
      SetLevel("MR3", (r2+r3)/2,    MidPivotsColor, LineStyle, LineThickness, startofday);
      SetLevel("MR2", (r1+r2)/2,    MidPivotsColor, LineStyle, LineThickness, startofday);
      SetLevel("MR1", (p+r1)/2,     MidPivotsColor, LineStyle, LineThickness, startofday);
      SetLevel("MS1", (p+s1)/2,     MidPivotsColor, LineStyle, LineThickness, startofday);
      SetLevel("MS2", (s1+s2)/2,    MidPivotsColor, LineStyle, LineThickness, startofday);
      SetLevel("MS3", (s2+s3)/2,    MidPivotsColor, LineStyle, LineThickness, startofday);
      SetLevel("MS4", (s3+s4)/2,    MidPivotsColor, LineStyle, LineThickness, startofday);
      SetLevel("MS5", (s4+s5)/2,    MidPivotsColor, LineStyle, LineThickness, startofday);

      gvname=Symbol()+"mr5";
   	gvval=(r4+r5)/2;
   	gvval=NormalizeDouble(gvval,digits);
   	GlobalVariableSet(gvname,gvval);

   	gvname=Symbol()+"mr4";
   	gvval=(r3+r4)/2;
   	gvval=NormalizeDouble(gvval,digits);
   	GlobalVariableSet(gvname,gvval);

   	gvname=Symbol()+"mr3";
   	gvval=(r2+r3)/2;
   	gvval=NormalizeDouble(gvval,digits);
   	GlobalVariableSet(gvname,gvval);

   	gvname=Symbol()+"mr2";
   	gvval=(r1+r2)/2;
   	gvval=NormalizeDouble(gvval,digits);
   	GlobalVariableSet(gvname,gvval);

   	gvname=Symbol()+"mr1";
   	gvval=(p+r1)/2;
   	gvval=NormalizeDouble(gvval,digits);
   	GlobalVariableSet(gvname,gvval);

   	gvname=Symbol()+"ms1";
   	gvval=(p+s1)/2;
   	gvval=NormalizeDouble(gvval,digits);
   	GlobalVariableSet(gvname,gvval);

   	gvname=Symbol()+"ms2";
   	gvval=(s1+s2)/2;
   	gvval=NormalizeDouble(gvval,digits);
   	GlobalVariableSet(gvname,gvval);

   	gvname=Symbol()+"ms3";
   	gvval=(s2+s3)/2;
   	gvval=NormalizeDouble(gvval,digits);
   	GlobalVariableSet(gvname,gvval);

   	gvname=Symbol()+"ms4";
   	gvval=(s3+s4)/2;
   	gvval=NormalizeDouble(gvval,digits);
   	GlobalVariableSet(gvname,gvval);

   	gvname=Symbol()+"ms5";
   	gvval= (s4+s5)/2;
   	gvval=NormalizeDouble(gvval,digits);
   	GlobalVariableSet(gvname,gvval);
   }

   //------ Comment for upper left corner
   if (ShowComment) {
      string comment= " ";      
      comment= comment + "-- Good luck with your trading! ---\n";
      comment= comment + "Range: Yesterday "+DoubleToStr(MathRound(q/Point),0) 
                       +" pips, Today "+DoubleToStr(MathRound(d/Point),0)+" pips" + "\n";
      comment= comment + "Highs: Yesterday "+DoubleToStr(yesterday_high,Digits)  
                       +", Today "+DoubleToStr(today_high,Digits) +"\n";
      comment= comment + "Lows:  Yesterday "+DoubleToStr(yesterday_low,Digits)  
                       +", Today "+DoubleToStr(today_low,Digits)  +"\n";
      comment= comment + "Close: Yesterday "+DoubleToStr(yesterday_close,Digits) + "\n";
   // comment= comment + "Pivot: " + DoubleToStr(p,Digits) + ", S1/2/3: " + DoubleToStr(s1,Digits)
   //                  + "/" + DoubleToStr(s2,Digits) + "/" + DoubleToStr(s3,Digits) + "\n" ;
   // comment= comment + "Fibos: " + DoubleToStr(yesterday_low + q*0.382, Digits) + ", " 
   //                  + DoubleToStr(yesterday_high - q*0.382,Digits) + "\n";     
      Comment(comment); 
   }
   return(0);
} 
//+------------------------------------------------------------------+
//| Compute index of first/last bar of yesterday and today           |
//+------------------------------------------------------------------+
void ComputeDayIndices(int tzlocal, int tzdest, int &idxfirstbaroftoday, int &idxfirstbarofyesterday, int &idxlastbarofyesterday)
{     
   int tzdiff= tzlocal - tzdest,
       tzdiffsec= tzdiff*3600,
       dayminutes= 24 * 60,
       barsperday= dayminutes/Period();
   
   int dayofweektoday= TimeDayOfWeek(Time[0] - tzdiffsec),  // what day is today in the dest timezone?
       dayofweektofind= -1; 
   // due to gaps in the data, and shift of time around weekends (due 
   // to time zone) it is not as easy as to just look back for a bar 
   // with 00:00 time  
   idxfirstbaroftoday= 0;
   idxfirstbarofyesterday= 0;
   idxlastbarofyesterday= 0;      
   switch (dayofweektoday) {
      case 6: // sat
      case 0: // sun
      case 1: // mon
            dayofweektofind= 5; // yesterday in terms of trading was previous friday
            break;           
      default:
            dayofweektofind= dayofweektoday -1; 
            break;
   }           
   // search  backwards for the last occrrence (backwards) of the day today (today's first bar)
   for (int i=1; i<=barsperday+1; i++) {
      datetime timet= Time[i] - tzdiffsec;
      if (TimeDayOfWeek(timet)!=dayofweektoday) {
         idxfirstbaroftoday= i-1;
         break;
      }
   }
   // search  backwards for the first occrrence (backwards) of the weekday we are looking for (yesterday's last bar)
   for (int j= 0; j<=2*barsperday+1; j++) {
      datetime timey= Time[i+j] - tzdiffsec;
      if (TimeDayOfWeek(timey)==dayofweektofind) {  // ignore saturdays (a Sa may happen due to TZ conversion)
         idxlastbarofyesterday= i+j;
         break;
      }
   }
   // search  backwards for the first occurrence of weekday before yesterday (to determine yesterday's first bar)
   for (j= 1; j<=barsperday; j++) {
      datetime timey2= Time[idxlastbarofyesterday+j] - tzdiffsec;
      if (TimeDayOfWeek(timey2)!=dayofweektofind) {  // ignore saturdays (a Sa may happen due to TZ conversion)
         idxfirstbarofyesterday= idxlastbarofyesterday+j-1;
         break;
      }
   } 
}
//+------------------------------------------------------------------+
//| Helper                                                           |
//+------------------------------------------------------------------+
void SetLevel(string text, double level, color col1, int linestyle, int thickness, datetime startofday)
{
   int digits= Digits; 
   string labelname= "[PIVOT] " + text + " Label", linename= "[PIVOT] " + text + " Line", pricelabel; 
   // create or move the horizontal line   
   int Z = OBJ_TREND;  //startofday = startofday - (Period()/240);//fudgeforce lines on Sunday 4hr chart
   if (FullLinesWithMarginLabels == true){Z = OBJ_HLINE; startofday=0;}
   if (ObjectFind(linename) != 0) 
       {
       ObjectCreate(linename, Z, 0, startofday, level, Time[0],level);
       ObjectSet   (linename, OBJPROP_STYLE, linestyle);
       ObjectSet   (linename, OBJPROP_COLOR, col1);
       ObjectSet   (linename, OBJPROP_WIDTH, thickness);
       }
   else
       {
       ObjectMove  (linename, 1, Time[0],level);
       ObjectMove  (linename, 0, startofday, level);
       }
   // put a label on the line   
   if (MoveLabel_LR_IncrDecr <=0) {MoveLabel_LR_IncrDecr = 1;}
   int A = Time[MoveLabel_LR_IncrDecr]; int B = startofday; int C = A;
  
   if (FullLinesWithMarginLabels == false)
       {       
       if(A<B+(7*Period()*60)){C=B+( 7*Period()*60);} //force time for label inside "today" period
       }
       //Comment(A+"   "+B+"   "+C+"   "+  (7*Period()*60) );  //debugging....
   if (ObjectFind(labelname) != 0) 
       {
       //ObjectCreate(labelname, OBJ_TEXT, 0, MathMin(Time[BarForLabels], startofday + 2*Period()*60), level);
       //ObjectCreate(labelname, OBJ_TEXT, 0, Time[MoveLabel_LR_IncrDecr], level);
       ObjectCreate(labelname, OBJ_TEXT, 0, C, level);
       }
   else 
       {
       //ObjectMove(labelname, 0, MathMin(Time[BarForLabels], startofday+2*Period()*60), level);
       //ObjectMove(labelname, 0, Time[MoveLabel_LR_IncrDecr] , level);
       ObjectMove(labelname, 0, C, level);
       }
   pricelabel= " " + text;
   if (LineLabelShowsPrice && StrToInteger(text)==0) pricelabel= pricelabel + ": "+DoubleToStr(level, Digits);   
   ObjectSetText(labelname, pricelabel, LineLabelFontSize , "Arial", PivotLinesLabelColor); //line labels
}
//+-------------------------------------------------------------------------------------------+
//| Helper=draws vertical timelines & gets "yesterday/today" from elsewhere and displays them.|                                                         |
//+-------------------------------------------------------------------------------------------+
void SetTimeLine(string objname, string text, int idx, color col1, double vleveltext) 
{
   string name= "[PIVOT] " + objname;
   int x= Time[idx];
   if(ShowPeriodSeparators == true) 
   {
   if (ObjectFind(name) != 0) { ObjectCreate(name, OBJ_TREND, 0, x, 0, x, 100);}
   else { ObjectMove(name, 0, x, 0); ObjectMove(name, 1, x, 100); }
 //  }  
   ObjectSet(name, OBJPROP_STYLE, SeparatorLineStyle_01234);
   ObjectSet(name, OBJPROP_COLOR, PeriodSeparatorLines);//vertical separators
   ObjectSet(name, OBJPROP_WIDTH, 1); 
   if (ObjectFind(name + " Label") != 0) {ObjectCreate(name + " Label", OBJ_TEXT, 0, x, vleveltext);}
   else {ObjectMove(name + " Label", 0, x, vleveltext);           
   ObjectSetText(name + " Label", text, 10, "Arial Bold", col1);} //"yesterday" and "today" vleveltext
   }
}
//------------------------End Program-----------------------------------------------------------------